{{

  **************************************************************************************
  **            Britt Tool, Inc -- Data Collection System v1.0                        **
  **            By: Rick Finkbiner & Corey Britton                                    **
  **            March 11, 2008                                                        **
  **************************************************************************************

}}

CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                                                                   

  ' Keyboard Connection
  DPin            =             0
  ClkPin          =             1
  ' PINK Connections
  NetRX           =             3
  NetTX           =             4
  ' RFID Connections
  RFID_Enable     =             5
  RFID_Serial     =             6 
  ' Buttons
  ClockInButton   =             8
  ClockOutButton  =             9
  CloseOpButton   =             10
  YesButton       =             11             
  NoButton        =             12
  ' Pins 16-23 are connected to the vga monitor per the demo board layout
  VGA_Start       =             16

  ' Monitor Clear Screen Command
  CLR             =             $00
    
OBJ

  BarCode          :               "keyboard"                   ' Barcode scanner object
  Monitor          :               "vga_text"                   ' VGA monitor object
  Serial           :               "Simple_Serial"              ' Serial communication object
  Network          :               "PinkV2"                     ' Network communication object
  
VAR

  LONG          Pink_Cog                                        ' ID for the PINK cog
  LONG          Timer_Cog
  LONG          stack[160]                                       ' stack space for the PINK
  BYTE          TX_Data                                         ' Transmit data flag
  BYTE          RFID_Code[11]                                   ' RFID Code array
  BYTE          Job[20]                                         ' Job number array
  BYTE          OpNumber[15]                                    ' Op number array
  BYTE          PinkMessage[50]                                 ' PINK UDP communication variable
  BYTE          ButtonCode                                      ' Flag that records what button was pressed
  BYTE          TimerFlag                                       ' Sets the auto reboot timer on and off
  BYTE          DisplayOK                                       ' Allows button presses to be read
        
PUB Main

  SystemINI                                                     ' Sets up the system then scans for user input
  repeat  
    if ina[ClockInButton] == 0
      ButtonCode := "0"                                         ' Clock in job flag
      GetData
    elseif ina[ClockOutButton] == 0
      ButtonCode := "1"                                         ' Clock out job flag
      GetData
    elseif ina[CloseOpButton] == 0
      ButtonCode := "2"                                         ' Close out job flag
      GetData
    
PRI GetData

  DisplayOK := 1                                                ' "walks" through the scan in procedure
  WelcomeMsg
  TimerFlag := 1
  RFIDScan
  TimerFlag := 0
  DisplayRFID
  TimerFlag := 1
  JobScan
  TimerFlag := 0
  DisplayJob
  TimerFlag := 1
  OpScan
  TimerFlag := 0
  DisplayOp
  TX_Data := 1
  repeat 
    waitcnt(clkfreq / 30 + cnt)
    if DisplayOK == 0
      quit
      
PRI SystemINI

  Monitor.start(VGA_Start)                                             ' Sets up the Monitor, network, and buttons before the program begins
  DisplayBTI
  TX_Data := 0
  TimerFlag := 0
  DisplayOK := 0
  Pink_Cog := cognew(Pink_Process, @stack[0]) + 1
  Timer_Cog := cognew(ResetChip, @stack[100]) + 1
  dira[ClockInButton] := 0
  dira[ClockOutButton] := 0
  dira[CloseOpButton] := 0
  dira[YesButton] := 0
  dira[NoButton] := 0

PRI WelcomeMsg

  waitcnt(clkfreq / 20 + cnt)                                   ' Begins the Job scan procedure
  Monitor.out(CLR)
  Monitor.str(String(13))
  Case ButtonCode
    "0" : Monitor.str(string("     Clock in to a job.", 13, 13))
    "1" : Monitor.str(string("     Clock out of a job.", 13, 13))
    "2" : Monitor.str(string("  Close out a job operation.", 13, 13))
  Monitor.str(string("  Please scan time card now."))
  
PRI DisplayRFID

  Monitor.str(string(13, "   RFID: "))                          ' Displays the RFID data as requests the Job number
  Monitor.str(@RFID_Code)
  Monitor.str(string(13, "  Please scan job now."))
  
PRI DisplayJob

  Monitor.str(string(13, "   Job: "))                           ' Displays the Job number and requests the Operation number
  Monitor.str(@Job)
  Monitor.str(string(13, "  Please scan operation now."))

PRI DisplayOp

  Monitor.str(string(13, "   Operation: "))                     ' Displays the Job Operation information
  Monitor.str(@OpNumber)

PRI DisplayBTI
                                                                
  Monitor.out(CLR)                                              ' Displays the "Welcome" screen
  Monitor.str(string(13, 13, 13))
  Monitor.str(string("       Britt Tool, Inc.", 13))
  Monitor.str(string("     Data Collections v1.0", 13))
  Monitor.str(string("   Press any key to continue."))
  DisplayOK := 0
  
PRI DisplayMsg(UDP_Msg)

  waitcnt(clkfreq / 20 + cnt) 
  Monitor.out(CLR)                                              ' Diaplays any messages the PINK gets via UDP
  Monitor.str(string(13, 13, 13, "     "))
  Monitor.str(UDP_Msg + 1)
  waitcnt(clkfreq * 5 + cnt)
  DisplayBTI
    
PRI RFIDScan | Index, Baud
 
  dira[RFID_Enable] := 1                                        ' Sets the RFID ports for communication
  dira[RFID_Serial] := 0 
  outa[RFID_Enable] := 0
  Baud := 2400                                                  ' Sets the serial baud rate to 2400
  Serial.Start(RFID_Serial, -1, Baud)                              ' Starts Serial communication                                                        ' Loops until you have data
  WaitForSignal
  repeat Index from 0 to 9                                      ' Load the RFID data into the buffer
    RFID_Code[Index] := Serial.rx
  outa[RFID_Enable] := 1                                        ' Stops the RFID reader from getting data
  Serial.Stop
  RFID_Code[10] := 0
  if (RFID_Code[9] =< 47) OR (RFID_Code[9] => 91)
    BadRFID
    reboot

PRI BadRFID

  Monitor.setcolors(@palette[0])
  Monitor.out(CLR)                                              ' Displays the "Bad RFID" screen
  Monitor.str(string(13, 13, 13))
  Monitor.str(string("    The RFID ("))
  Monitor.str(@RFID_Code)
  Monitor.str(string(")", 13))
  Monitor.str(string("  is incorrect.  Please try", 13))
  Monitor.str(string("   this operation again."))
  waitcnt(clkfreq * 5 + cnt)
  
PRI WaitForSignal

  repeat                                                        ' Loops until you have data
    if Serial.rx == $0A                                         ' Exit the loop when you get the start code ($0A)
      quit

PRI JobScan | Index

  BarCode.start(DPin, ClkPin)                                   ' Scans the Job barcode and writes it to the Job array
  bytefill(@Job, 0, 20)
  repeat Index from 0 to 19
    Job[Index] := BarCode.getkey 
    if Job[Index] == 13
      Job[Index] := 0
      quit
  if (Job[0] => 48) AND (Job[0] =< 57)
    BadJob
    reboot
    
PRI BadJob

  Monitor.setcolors(@palette[0])
  Monitor.out(CLR)                                               ' Displays the "Bad Job" screen
  Monitor.str(string(13, 13, 13))
  Monitor.str(string("    The Job Number ("))
  Monitor.str(@Job)
  Monitor.str(string(")", 13))
  Monitor.str(string("  is incorrect.  Please try", 13))
  Monitor.str(string("   this operation again."))
  waitcnt(clkfreq * 5 + cnt)
  
PRI OpScan | Index

  BarCode.start(DPin, ClkPin)                                   ' Scans the Op number and writes it to the OpNumber array
  bytefill(@OpNumber, 0, 15)
  repeat Index from 0 to 14
    OpNumber[Index] := BarCode.getkey 
    if OpNumber[Index] == 13
      OpNumber[Index] := 0
      quit
  if OpNumber[0] => 58
    BadOp
    reboot
    
PRI BadOp

  Monitor.setcolors(@palette[0]) 
  Monitor.out(CLR)                                              ' Displays the "Bad Op" screen
  Monitor.str(string(13, 13, 13))
  Monitor.str(string("     The Op Number ("))
  Monitor.str(@OpNumber)
  Monitor.str(string(")", 13))
  Monitor.str(string("  is incorrect.  Please try", 13))
  Monitor.str(string("   this operation again."))
  waitcnt(clkfreq * 5 + cnt)
  
PRI Pink_Process
   
   Network.PinkV2("0",NetTX,NetRX)                              ' Sets the network card object ini information
  repeat                                                        ' Loops until the network is connected
    Network.getStatus
  until Network.isConnected
  repeat
    Network.getStatus
    if Network.isUdp
      ReadData                                                  ' Checks the network card data buffer for data and                                                  ' sends any data in the RFID data buffer
    if TX_Data == 1 
      SendData
      TX_Data := 0

PRI ReadData

  Network.readVar(Network#Nb_varBM, @PinkMessage)               ' Reads and UDP messages sent to the PINK
  waitcnt(clkfreq / 20 + cnt)
  if PinkMessage[0] == "0"
    DisplayMsg(@PinkMessage)
  elseif PinkMessage[0] == "1"
    DisplayQst(@PinkMessage)
     
PRI SendData

  repeat
    waitcnt(clkfreq + cnt)                                      ' Sends any data in the KeyData buffer through the PINK object UDP
    Network.getStatus                                           ' system
  until Network.isReady
  ClockMessage
  Network.writeVar(Network#Nb_varBM, @PinkMessage)
  Network.sendUdp
  
PRI ClockMessage                 | Index, StopPoint

  bytefill(@PinkMessage,0, 50)                                  ' Creates the PINK data stream to send out to the server
  repeat Index from 0 to 9                                      ' format: RFID_Job_OP_ButtonCode
    PinkMessage[Index] := RFID_Code[Index]
  PinkMessage[10] := "_"
  repeat Index from 0 to 19
    if Job[Index] == 0
      StopPoint := Index + 12
      PinkMessage[Index + 11] := "_"
      quit
    PinkMessage[Index + 11] := Job[Index]
  repeat Index from 0 to 14
    if Opnumber[Index] == 0
      PinkMessage[Index + StopPoint] := "_"
      StopPoint += Index + 1
      quit
    PinkMessage[Index + StopPoint] := OpNumber[Index]
  PinkMessage[StopPoint] := ButtonCode
  
PRI DisplayQst(Qstn)   | TempCnt

  waitcnt(clkfreq / 20 + cnt)   
  Monitor.out(CLR)                                              ' Displays a question sent from the server to the Prop
  Monitor.str(string(13, 13, 13, "     "))                      ' if gives you 9 seconds to answer before defaults to "no"
  Monitor.str(Qstn + 1)
  ButtonCode := "N" 
  TempCnt := cnt + (clkfreq * 9)
  repeat while TempCnt > cnt   
    if ina[YesButton] == 0 
      ButtonCode := "Y"
      quit
    elseif ina[NoButton] == 0 
      ButtonCode := "N"
      quit
  TX_Data := 1

PRI ResetChip 

  repeat
    repeat
      if TimerFlag == 1
        quit
    repeat 60
      waitcnt(clkfreq / 2 + cnt)
      if TimerFlag == 0
        quit
    if TimerFlag == 1
      reboot

DAT

                        '        fore   back
                        '         RGB    RGB
palette                 byte    %%333, %%300    '0    white / red   warning messages
   